@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 # 05.12.2005, c
 from __future__ import division, print_function, absolute_import
+import sys
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
@@ -11,6 +12,12 @@ def configuration(parent_package='',top_path=None):
 
     umf_info = get_info('umfpack', notfound_action=1)
 
+    ## The following addition is needed when linking against a umfpack built
+    ## from the latest SparseSuite. Not (strictly) needed when linking against
+    ## the version in the ubuntu repositories.
+    if not sys.platform in ('darwin', 'win32'):
+        umf_info['libraries'].insert(0, 'rt')
+
     umfpack_i_file = config.paths('umfpack.i')[0]
 
     def umfpack_i(ext, build_dir):
@@ -21,7 +28,7 @@ def configuration(parent_package='',top_path=None):
     build_info = {}
     dict_append(build_info, **umf_info)
     dict_append(build_info, **blas_info)
-
+    
     config.add_extension('__umfpack',
                          sources=[umfpack_i],
                          depends=['umfpack.i'],
